#include <octave/oct.h>
#include <octave/CMatrix.h>
#include <octave/Matrix.h>
#include <map>


/**
 * global variable holding the handles to the private library
 * ************************************************************************** */

static std::map<int,void*>  mylib_handles;


/**
 *
 * ************************************************************************** */
DEFUN_DLD(mylib_init, args, ,
        "mylib_init"
        )
{
    int     key;
    void    *handle;

    /* initialize the library, dummy code */

    handle = malloc(sizeof(int));

    /* find next valid key and store the handles */

    for (   key = 0;
            ! mylib_handles.insert(std::pair<int,void*>(key, handle)).second;
            key++);

    return octave_value(key);
}


/**
 *
 * ************************************************************************** */
DEFUN_DLD(mylib_destroy, args, ,
        "mylib_destroy"
        )
{
    std::map<int,void*>::iterator   it;

    if (args.length() != 1)
        print_usage();

    /* get key argument */

    const int key = args(0).int_value();

    /* find handle corresponding to key and release the library */

    it = mylib_handles.find(key);
    if (it != mylib_handles.end()) {
        void *h = it->second;
        mylib_handles.erase(it);

        free(h);
    }
    else {
        error("mylib_destroy: handle '%d' does not exist!", key);
    }

    return octave_value_list();
}


/**
 *
 * ************************************************************************** */
DEFUN_DLD(mylib_process, args, ,
        "mylib_process"
        )
{
    std::map<int,void*>::iterator   it;

    /* check arguments */

    if (args.length() != 1)
        print_usage();

    const int key = args(0).int_value();

    /* find handle corresponding to key */

    it = mylib_handles.find(key);
    if (it == mylib_handles.end()) {
        error("mylib_process: handle '%d' does not exist!", key);
    }
    void *h = it->second;

    /* do processing */

    /* ... */

    return octave_value();
}
